/*
 * The graphical shell used here is made at University of Virginia by
 * Nathan Hoobler, Cliff Woolley, Benjamin Cummings, and some contributions from
 * Dave Luebke and Yannick Loitiere
 *
 * Implemented functions for use with IBMR by Abhinav Dayal
 */

#include "console.h"
#include "shell.h"
#include "externs.h"

// this is a sample function to show how to declare commands
// first argument is command, and second is the help for it
gsh_DECLARE_CMD_EX(something,"just a test") 
{
    // Note the use of argc and argv as if this was a main() function
    int i;
    gsh_Printf("Arguments supplied: ");
    for (i=0; i<argc; i++)
    {
        gsh_Printf("%s ", argv[i]); 
    }
    gsh_Printf("\n"); 
}

gsh_DECLARE_CMD_EX(snapshot,"take snapshot of this window")
{
	if(save_screenshot(SCREENSHOT))
		gsh_Printf("Could not save to file %s.\n", SCREENSHOT);
	else
		gsh_Printf("Current display saved to file %s.\n", SCREENSHOT);
}

gsh_DECLARE_CMD_EX(reset,"reset the mesh to initial values")
{
	myMesh.reset();
	gsh_Printf("Mesh reset to initial state");
}

gsh_DECLARE_CMD_EX(applyH,"Apply the global transformation matrix H to the mesh")
{
	int j;
	myMesh.ApplyTransform(H);
	gsh_Printf("Applied the following matrix transformation:");
	for(j=0; j< 3; j++) 
	{
       			gsh_Printf("%f  %f  %f",H.colrow[0][j], H.colrow[1][j], H.colrow[2][j]);
	}
}

gsh_DECLARE_CMD_EX(setIncrement,"set how much to move the mesh up/down")
{
    // Note the use of argc and argv as if this was a main() function
    if(argc!=2)
	{
		gsh_Printf("syntax: setIncrement <float value>");
		return;
	}
	increment = atof(argv[1]);
	gsh_Printf("setting increment to %f", increment);
}

gsh_DECLARE_CMD_EX(loadBMP,"load a custom BMP image into texture") 
{
    // Note the use of argc and argv as if this was a main() function
    if(argc!=2)
	{
		gsh_Printf("syntax: loadBMP path/to/file.bmp");
		return;
	}
	LoadTexture(argv[1]);
	gsh_Printf("%s", gsh_message);
}

gsh_DECLARE_CMD_EX(toggleTexture,"Toggle texture On/Off" )
{
	enableTexture = !enableTexture;
	if(enableTexture)
	{
		gsh_Printf("Texture Mapping turned ON");
		glEnable(GL_TEXTURE_2D);
	}
	else
	{
		gsh_Printf("Texture Mapping turned OFF");
		glDisable(GL_TEXTURE_2D);
	}
}

gsh_DECLARE_CMD_EX(toggleDisplayMode,"Toggle display mode between LINE/VERICES/FILL")
{
// toggle between lines, points and fill mode
	switch(mesh_display_mode)
	{
		case LINE: 
			mesh_display_mode = VERTEX;
			gsh_Printf("Mesh Display Mode: VERTEX");
			break;
		case VERTEX: 
			mesh_display_mode = FILL;
			gsh_Printf("Mesh Display Mode: FILL");
			break;
		case FILL: 
			mesh_display_mode = LINE;
			gsh_Printf("Mesh Display Mode: LINE");
			break;
	};
}

			
void init_gcon()
{
	gcon_Init(512, 0.8f, 0.8f, 0.8f);
    gcon_SetBackgroundColor(0.0f, 0.3f, 0.0f, BACKGROUND_ALPHA);

	// initialze glControl, this enables glDisable etc. on our shell
    gshglcontrol_Init();

	// register all our commands into the shell
	gsh_REGISTER(something); 
	gsh_REGISTER(loadBMP);
	gsh_REGISTER(reset);
	gsh_REGISTER(applyH);
	gsh_REGISTER(toggleDisplayMode);
	gsh_REGISTER(toggleTexture);
	gsh_REGISTER(snapshot);
	gsh_REGISTER(setIncrement);

	// register key bindings
    gsh_ExecLine("bind esc quit");
    gsh_ExecLine("bind q quit");
	gsh_ExecLine("bind r reset");
	gsh_ExecLine("bind h applyH");
	gsh_ExecLine("bind s snapshot");
	gsh_ExecLine("bind t toggleTexture");
	gsh_ExecLine("bind m toggleDisplayMode");

	// print the initial message
	gsh_Printf("gcon: by - Nathan Hobler, Cliff Woolley, Benjamin Cummings");
	gsh_Printf("           Dave Luebke & Yannick Loitiere at Univ. Virginia");
	gsh_Printf("Press ESC or Q to quit");
    gsh_Printf("Press ` for shell. Say \"help\" or \"bind list\" there for options\n");
}

